<?php
namespace Ak\Orm\Service;

use Ak\Orm\Type\HydratedObject;
use Doctrine\DBAL\Connection;
use Doctrine\DBAL\Types\Type;
use Doctrine\ORM\EntityManager;
use Doctrine\ORM\EntityManagerInterface;
use Doctrine\ORM\ORMException;
use Interop\Container\ContainerInterface;
use Laminas\ServiceManager\Exception\ServiceNotCreatedException;
use Laminas\ServiceManager\Factory\FactoryInterface;

/**
 * Stvara novu instancu entity managera.
 *
 * @package Ak\Orm\Service
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/orm
 */
final class EntityManagerFactory implements FactoryInterface
{
    /** @var array */
    private $config;
    /** @var bool */
    private $isDevelopment;
    /** @var bool */
    private static $typesRegistered = false;

    /**
     * Stvara entity manager.
     *
     * @param  ContainerInterface $container
     * @param  string $requestedName
     * @param  null|array $options
     *
     * @return EntityManagerInterface
     * @throws \Doctrine\DBAL\DBALException
     */
    public function __invoke(ContainerInterface $container, $requestedName, array $options = null)
    {
        if (isset($options['connection']) && $options['connection'] instanceof Connection) {
            $connection = $options['connection'];
        } else {
            $connection = $this->getConnection($container);
        }

        $configuration = $container->get('orm.configuration');
        $events = $container->get('orm.events');

        try {
            $em = EntityManager::create($connection, $configuration, $events);
            $this->registerTypes();
        } catch (ORMException $e) {
            throw new ServiceNotCreatedException('Ne mogu stvoriti Entity Manager.', 0, $e);
        }

        return $em;
    }

    /**
     * Vraća relevantnu konfiguraciju.
     *
     * @param ContainerInterface $container
     * @return array
     */
    private function getConfig(ContainerInterface $container): array
    {
        if (null === $this->config) {
            $config = $container->get('config');
            $this->isDevelopment = $config['development'] ?? false;
            $this->config = $config['orm'];
        }

        return $this->config;
    }

    /**
     * Dohvaća konekciju na bazu.
     *
     * @param ContainerInterface $container
     * @return Connection
     */
    private function getConnection(ContainerInterface $container): Connection
    {
        $config = $this->getConfig($container);
        $connection = $config['connection'] ?? 'db';

        return $container->get($connection);
    }

    /**
     * Registrira custom tipove.
     * @throws \Doctrine\DBAL\DBALException
     */
    private function registerTypes(): void
    {
        if (!self::$typesRegistered) {
            Type::addType(HydratedObject::HYDRATED_OBJECT, HydratedObject::class);
            self::$typesRegistered = true;
        }
    }
}
