<?php
namespace AkTest\Orm\Filter;

use Ak\Filter\ArrayFilter;
use Ak\Orm\Filter\Entity;
use AkTest\Orm\AbstractTestCase;
use AkTest\Orm\Entity\ConcreteEntity;
use Doctrine\ORM\EntityManagerInterface;
use PHPUnit\Framework\MockObject\MockObject;
use Laminas\Filter\FilterPluginManager;
use Laminas\InputFilter\InputFilter;
use Laminas\InputFilter\InputFilterPluginManager;
use Laminas\ServiceManager\PluginManagerInterface;

/**
 * Class EntityTest
 *
 * @package AkTest\Orm\Filter
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/orm
 */
final class EntityTest extends AbstractTestCase
{
    /**
     * @return FilterPluginManager
     */
    protected function getFilterManager(): FilterPluginManager
    {
        $container = $this->getContainer();
        return $container->get(FilterPluginManager::class);
    }

    public function testIsRegistered()
    {
        $plugins = $this->getFilterManager();

        $this->assertContainerHasService($plugins, [
            'entity',
            Entity::class,
        ], Entity::class, false);
    }

    public function testCanPassEntityClassInConstructor()
    {
        $filter = new Entity(ConcreteEntity::class);
        $this->assertEquals(ConcreteEntity::class, $filter->getEntityClass());
    }

    public function testCanPassEntityClassAsOption()
    {
        $filter = new Entity([
            'entity_class' => ConcreteEntity::class,
        ]);

        $this->assertEquals(ConcreteEntity::class, $filter->getEntityClass());
    }

    public function testEntityManagerIsInjected()
    {
        $filters = $this->getFilterManager();
        /** @var Entity $filter */
        $filter = $filters->get('entity');

        $this->assertSame($this->getEntityManager(), $filter->getEntityManager());
    }

    /**
     * @expectedException \Laminas\Filter\Exception\RuntimeException
     */
    public function testWillThrowExceptionForNonNumericValues()
    {
        $filter = new Entity();
        $filter('text');
    }

    /**
     * @expectedException \Laminas\Filter\Exception\RuntimeException
     */
    public function testWillThrowExceptionIfEntityManagerNotSet()
    {
        $filter = new Entity();
        $filter(1);
    }

    /**
     * @expectedException \Laminas\Filter\Exception\RuntimeException
     */
    public function testWillThrowExceptionIfEntityClassNotSet()
    {
        $filter = new Entity();
        $filter->setEntityManager($this->getEntityManager());

        $filter(1);
    }

    public function testWillFilterTheValue()
    {
        $entity = $this->createMock(ConcreteEntity::class);
        /** @var EntityManagerInterface|MockObject $em */
        $em = $this->createMock(EntityManagerInterface::class);
        $em->expects($this->once())->method('find')->with(ConcreteEntity::class, 1)->willReturn($entity);

        $filter = new Entity(ConcreteEntity::class);
        $filter->setEntityManager($em);

        $this->assertSame($entity, $filter(1));
    }

    public function testCanBeCreatedFromInputFilterSpec()
    {
        $container = $this->getContainer();
        /** @var PluginManagerInterface $manager */
        $manager = $container->get(InputFilterPluginManager::class);
        /** @var InputFilter $filter */
        $filter = $manager->get(InputFilter::class);

        $filter->add([
            'name' => 'field',
            'filters' => [[
                'name' => 'entity',
            ]],
        ]);

        $this->addToAssertionCount(1);
    }

    public function testCanCreateArrayFilterWithEntityFilter()
    {
        $container = $this->getContainer();
        /** @var FilterPluginManager $filters */
        $filters = $container->get(FilterPluginManager::class);
        /** @var ArrayFilter $filter */
        $filter = $filters->get('array', [
            'filter' => [
                'name' => 'entity',
                'options' => [
                    'entity_class' => Entity::class,
                ],
            ]
        ]);

        $this->assertInstanceOf(Entity::class, $filter->getFilter());
    }
}
